@@ -6,12 +6,11 @@ strides, dtype and size attributes similar to a NumPy ndarray.
 from __future__ import print_function, absolute_import, division
 import warnings
 import math
-import operator
 import numpy as np
-from functools import reduce
-from .ndarray import ndarray_device_allocate_head, ndarray_populate_head
+from .ndarray import (ndarray_populate_head, ArrayHeaderManager)
 from . import driver as _driver
 from . import devices
+from numba import dummyarray
 
 try:
     long
@@ -55,15 +54,23 @@ class DeviceNDArrayBase(object):
     def __init__(self, shape, strides, dtype, stream=0, writeback=None,
                  gpu_head=None, gpu_data=None):
         """
-        Arguments
+        Args
+        ----
 
-        shape: array shape.
-        strides: array strides.
-        dtype: data type as numpy.dtype.
-        stream: cuda stream.
-        writeback: Deprecated.
-        gpu_head: user provided device memory for the ndarray head structure
-        gpu_data: user provided device memory for the ndarray data buffer
+        shape
+            array shape.
+        strides
+            array strides.
+        dtype
+            data type as numpy.dtype.
+        stream
+            cuda stream.
+        writeback
+            Deprecated.
+        gpu_head
+            user provided device memory for the ndarray head structure
+        gpu_data
+            user provided device memory for the ndarray data buffer
         """
         if isinstance(shape, (int, long)):
             shape = (shape,)
@@ -72,6 +79,8 @@ class DeviceNDArrayBase(object):
         self.ndim = len(shape)
         if len(strides) != self.ndim:
             raise ValueError('strides not match ndim')
+        self._dummy = dummyarray.Array.from_desc(0, shape, strides,
+                                                 dtype.itemsize)
         self.shape = tuple(shape)
         self.strides = tuple(strides)
         self.dtype = np.dtype(dtype)
@@ -85,8 +94,10 @@ class DeviceNDArrayBase(object):
         else:
             self.alloc_size = _driver.device_memory_size(gpu_data)
 
+        self.gpu_mem = ArrayHeaderManager(devices.get_context())
+
         if gpu_head is None:
-            gpu_head = ndarray_device_allocate_head(self.ndim)
+            gpu_head = self.gpu_mem.allocate(self.ndim)
             ndarray_populate_head(gpu_head, gpu_data, self.shape,
                                   self.strides, stream=stream)
         self.gpu_head = gpu_head
@@ -94,22 +105,16 @@ class DeviceNDArrayBase(object):
 
         self.__writeback = writeback # should deprecate the use of this
 
-        # define the array interface to work with numpy
-        #
-        # XXX: problem with data being accessed.
-        #      is NULL pointer alright?
-        #
-        #        self.__array_interface__ = {
-        #            'shape'     : self.shape,
-        #            'typestr'   : self.dtype.str,
-        #            'data'      : (0, True),
-        #            'version'   : 3,
-        #        }
-
+    def __del__(self):
+        try:
+            self.gpu_mem.free(self.gpu_head)
+        except:
+            pass
 
     @property
     def device_ctypes_pointer(self):
-        "Returns the ctypes pointer to the GPU data buffer"
+        """Returns the ctypes pointer to the GPU data buffer
+        """
         return self.gpu_data.device_ctypes_pointer
 
     def copy_to_device(self, ary, stream=0):
@@ -165,10 +170,10 @@ class DeviceNDArrayBase(object):
         self.copy_to_host(self.__writeback, stream=stream)
 
     def split(self, section, stream=0):
-        '''Split the array into equal partition of the `section` size.
+        """Split the array into equal partition of the `section` size.
         If the array cannot be equally divided, the last section will be
         smaller.
-        '''
+        """
         if self.ndim != 1:
             raise ValueError("only support 1d array")
         if self.strides[0] != self.dtype.itemsize:
@@ -181,11 +186,8 @@ class DeviceNDArrayBase(object):
             end = min(begin + section, self.size)
             shape = (end - begin,)
             gpu_data = self.gpu_data.view(begin * itemsize, end * itemsize)
-            gpu_head = ndarray_device_allocate_head(1)
-            ndarray_populate_head(gpu_head, gpu_data, shape, strides,
-                                  stream=stream)
             yield DeviceNDArray(shape, strides, dtype=self.dtype, stream=stream,
-                                gpu_head=gpu_head, gpu_data=gpu_data)
+                                gpu_data=gpu_data)
 
     def as_cuda_arg(self):
         """Returns a device memory object that is used as the argument.
@@ -194,105 +196,80 @@ class DeviceNDArrayBase(object):
 
 
 class DeviceNDArray(DeviceNDArrayBase):
-    def _yields_f_strides_by_shape(self, shape=None):
-        """yields the f-contigous strides
-        """
-        shape = self.shape if shape is None else shape
-        itemsize = self.dtype.itemsize
-        yield itemsize
-        sum = 1
-        for s in shape[:-1]:
-            sum *= s
-            yield sum * itemsize
+    def is_f_contiguous(self):
+        return self._dummy.is_f_contig
 
-    def _yields_c_strides_by_shape(self, shape=None):
-        '''yields the c-contigous strides in
-        '''
-        shape = self.shape if shape is None else shape
-        itemsize = self.dtype.itemsize
-
-        def gen():
-            yield itemsize
-            sum = 1
-            for s in reversed(shape[1:]):
-                sum *= s
-                yield sum * itemsize
-
-        for i in reversed(list(gen())):
-            yield i
-
-    def is_f_contigous(self):
-        return all(i == j for i, j in
-                   zip(self._yields_f_strides_by_shape(), self.strides))
-
-    def is_c_contigous(self):
-        return all(i == j for i, j in
-                   zip(self._yields_c_strides_by_shape(),
-                       self.strides))
+    def is_c_contiguous(self):
+        return self._dummy.is_c_contig
 
     def reshape(self, *newshape, **kws):
-        '''reshape(self, *newshape, order='C'):
+        """reshape(self, *newshape, order='C'):
 
         Reshape the array and keeping the original data
-        '''
-        order = kws.pop('order', 'C')
-        if kws:
-            raise TypeError('unknown keyword arguments %s' % kws.keys())
-        if order not in 'CFA':
-            raise ValueError('order not C|F|A')
-            # compute new array size
-        if len(newshape) > 1:
-            newsize = reduce(operator.mul, newshape)
+        """
+        newarr, extents = self._dummy.reshape(*newshape, **kws)
+        cls = type(self)
+
+        if extents == [self._dummy.extent]:
+            return cls(shape=newarr.shape, strides=newarr.strides,
+                       dtype=self.dtype, gpu_data=self.gpu_data)
         else:
-            (newsize,) = newshape
-        if newsize != self.size:
-            raise ValueError("reshape changes the size of the array")
-        elif self.is_f_contigous() or self.is_c_contigous():
-            if order == 'A':
-                order = 'F' if self.is_f_contigous() else 'C'
-            if order == 'C':
-                newstrides = list(self._yields_c_strides_by_shape(newshape))
-            elif order == 'F':
-                newstrides = list(self._yields_f_strides_by_shape(newshape))
-            else:
-                assert False, 'unreachable'
-
-            ret = DeviceNDArray(shape=newshape, strides=newstrides,
-                                dtype=self.dtype, gpu_data=self.gpu_data)
-
-            return ret
-        else:
-            raise NotImplementedError("reshaping non-contiguous array requires"
-                                      "autojitting a special kernel to complete")
-
+            raise NotImplementedError("operation requires copying")
 
     def ravel(self, order='C', stream=0):
-        if order not in 'CFA':
-            raise ValueError('order not C|F|A')
-        elif self.ndim == 1:
-            return DeviceNDArray(shape=self.shape, strides=self.strides,
-                                 dtype=self.dtype, gpu_data=self.gpu_data)
-        elif (order == 'C' and self.is_c_contigous() or
-                          order == 'F' and self.is_f_contigous()):
-            # just flatten it
-            return DeviceNDArray(shape=self.size, strides=self.dtype.itemsize,
-                                 dtype=self.dtype, gpu_data=self.gpu_data,
-                                 stream=stream)
+        cls = type(self)
+        newarr, extents = self._dummy.ravel(order=order)
+
+        if extents == [self._dummy.extent]:
+            return cls(shape=newarr.shape, strides=newarr.strides,
+                       dtype=self.dtype, gpu_data=self.gpu_data,
+                       gpu_head=self.gpu_head, stream=stream)
+
         else:
-            raise NotImplementedError("this ravel operation requires "
-                                      "autojitting a special kernel to complete")
+            raise NotImplementedError("operation requires copying")
+
+    def __getitem__(self, item):
+        arr = self._dummy.__getitem__(item)
+        extents = list(arr.iter_contiguous_extent())
+        cls = type(self)
+        if len(extents) == 1:
+            newdata = self.gpu_data.view(*extents[0])
+
+            if dummyarray.is_element_indexing(item, self.ndim):
+                hostary = np.empty(1, dtype=self.dtype)
+                _driver.device_to_host(dst=hostary, src=newdata,
+                                       size=self._dummy.itemsize)
+                return hostary[0]
+            else:
+                return cls(shape=arr.shape, strides=arr.strides,
+                           dtype=self.dtype, gpu_data=newdata)
+        else:
+            newdata = self.gpu_data.view(*arr.extent)
+            return cls(shape=arr.shape, strides=arr.strides,
+                       dtype=self.dtype, gpu_data=newdata)
 
 
 class MappedNDArray(DeviceNDArrayBase, np.ndarray):
-    def device_setup(self, gpu_data, stream=0):
-        gpu_head = ndarray_device_allocate_head(self.ndim)
+    """
+    A host array that uses CUDA mapped memory.
+    """
 
+    def device_setup(self, gpu_data, stream=0):
+        self.gpu_mem = ArrayHeaderManager(devices.get_context())
+
+        gpu_head = self.gpu_mem.allocate(self.ndim)
         ndarray_populate_head(gpu_head, gpu_data, self.shape,
                               self.strides, stream=stream)
 
         self.gpu_data = gpu_data
         self.gpu_head = gpu_head
 
+    def __del__(self):
+        try:
+            self.gpu_mem.free(self.gpu_head)
+        except:
+            pass
+
 
 def from_array_like(ary, stream=0, gpu_head=None, gpu_data=None):
     "Create a DeviceNDArray object that is like ary."
@@ -303,17 +280,23 @@ def from_array_like(ary, stream=0, gpu_head=None, gpu_data=None):
                          gpu_data=gpu_data)
 
 
+errmsg_contiguous_buffer = ("Array contains non-contiguous buffer and cannot "
+                            "be transferred as a single memory region. Please "
+                            "ensure contiguous buffer with numpy "
+                            ".ascontiguousarray()")
+
+
+def sentry_contiguous(ary):
+    if not ary.flags['C_CONTIGUOUS'] and not ary.flags['F_CONTIGUOUS']:
+        if ary.ndim != 1 or ary.shape[0] != 1 or ary.strides[0] != 0:
+            raise ValueError(errmsg_contiguous_buffer)
+
+
 def auto_device(ary, stream=0, copy=True):
     if _driver.is_device_memory(ary):
         return ary, False
     else:
-        if not ary.flags['C_CONTIGUOUS'] and not ary.flags['F_CONTIGUOUS']:
-            if ary.ndim != 1 or ary.shape[0] != 1 or ary.strides[0] != 0:
-                raise ValueError(
-                    "Array contains non-contiguous buffer and cannot "
-                    "be transferred as a single memory region.  "
-                    "Please ensure contiguous buffer with numpy"
-                    ".ascontiguousarray()")
+        sentry_contiguous(ary)
         devarray = from_array_like(ary, stream=stream)
         if copy:
             devarray.copy_to_device(ary, stream=stream)
