@@ -40,6 +40,7 @@ class Interpreter(object):
         self.syntax_blocks = []
         self.dfainfo = None
         self._block_actions = {}
+        self.constants = {}
 
     def _fill_global_scope(self, scope):
         """TODO
@@ -69,10 +70,20 @@ class Interpreter(object):
         for b in utils.dict_itervalues(self.blocks):
             b.verify()
 
+    def init_first_block(self):
+        # Duplicate arguments so that these values can be casted into different
+        # types.
+        for aname in self.argspec.args:
+            aval = self.get(aname)
+            self.store(aval, aname)
+
     def _iter_inst(self):
-        for block in self.cfa.iterliveblocks():
+        for blkct, block in enumerate(self.cfa.iterliveblocks()):
             firstinst = self.bytecode[block.body[0]]
             self._start_new_block(firstinst)
+            if blkct == 0:
+                # Is first block
+                self.init_first_block()
             for offset, kws in self.dfainfo.insts:
                 inst = self.bytecode[offset]
                 self.loc = ir.Loc(filename=self.bytecode.filename,
@@ -122,7 +133,6 @@ class Interpreter(object):
 
                 self.store(target, phivar)
 
-
     def get_global_value(self, name):
         """
         Get a global value from the func_global (first) or
@@ -133,6 +143,12 @@ class Interpreter(object):
         except KeyError:
             return getattr(builtins, name, ir.UNDEFINED)
 
+    def get_closure_value(self, index):
+        """
+        Get a value from the cell contained in this function's closure.
+        """
+        return self.bytecode.func.__closure__[index].cell_contents
+
     @property
     def current_scope(self):
         return self.scopes[-1]
@@ -149,6 +165,10 @@ class Interpreter(object):
     def code_names(self):
         return self.bytecode.co_names
 
+    @property
+    def code_freevars(self):
+        return self.bytecode.co_freevars
+
     def _dispatch(self, inst, kws):
         assert self.current_block is not None
         fname = "op_%s" % inst.opname.replace('+', '_')
@@ -243,13 +263,17 @@ class Interpreter(object):
                 (), loc=self.loc)
         self.store(value=sliceinst, name=res)
 
-    def op_SLICE_0(self, inst, base, res, slicevar, indexvar):
+    def op_SLICE_0(self, inst, base, res, slicevar, indexvar, nonevar):
         base = self.get(base)
 
         slicegv = ir.Global("slice", slice, loc=self.loc)
         self.store(value=slicegv, name=slicevar)
 
-        index = ir.Expr.call(self.get(slicevar), (), (), loc=self.loc)
+        nonegv = ir.Const(None, loc=self.loc)
+        self.store(value=nonegv, name=nonevar)
+        none = self.get(nonevar)
+
+        index = ir.Expr.call(self.get(slicevar), (none, none), (), loc=self.loc)
         self.store(value=index, name=indexvar)
 
         expr = ir.Expr.getitem(base, self.get(indexvar), loc=self.loc)
@@ -306,13 +330,17 @@ class Interpreter(object):
         expr = ir.Expr.getitem(base, self.get(indexvar), loc=self.loc)
         self.store(value=expr, name=res)
 
-    def op_STORE_SLICE_0(self, inst, base, value, slicevar, indexvar):
+    def op_STORE_SLICE_0(self, inst, base, value, slicevar, indexvar, nonevar):
         base = self.get(base)
 
         slicegv = ir.Global("slice", slice, loc=self.loc)
         self.store(value=slicegv, name=slicevar)
 
-        index = ir.Expr.call(self.get(slicevar), (), (), loc=self.loc)
+        nonegv = ir.Const(None, loc=self.loc)
+        self.store(value=nonegv, name=nonevar)
+        none = self.get(nonevar)
+
+        index = ir.Expr.call(self.get(slicevar), (none, none), (), loc=self.loc)
         self.store(value=index, name=indexvar)
 
         stmt = ir.SetItem(base, self.get(indexvar), self.get(value),
@@ -380,6 +408,12 @@ class Interpreter(object):
         value = self.get(value)
         self.store(value=value, name=dstname)
 
+    def op_STORE_ATTR(self, inst, target, value):
+        attr = self.code_names[inst.arg]
+        sa = ir.SetAttr(target=self.get(target), value=self.get(value),
+                        attr=attr, loc=self.loc)
+        self.current_block.append(sa)
+
     def op_LOAD_ATTR(self, inst, item, res):
         item = self.get(item)
         attr = self.code_names[inst.arg]
@@ -396,6 +430,14 @@ class Interpreter(object):
         value = self.get_global_value(name)
         gl = ir.Global(name, value, loc=self.loc)
         self.store(gl, res)
+        self.constants[res] = value
+
+    def op_LOAD_DEREF(self, inst, res):
+        name = self.code_freevars[inst.arg]
+        value = self.get_closure_value(inst.arg)
+        # closure values are treated like globals
+        gl = ir.Global(name, value, loc=self.loc)
+        self.store(gl, res)
 
     def op_SETUP_LOOP(self, inst):
         assert self.blocks[inst.offset] is self.current_block
@@ -487,6 +529,11 @@ class Interpreter(object):
         expr = ir.Expr.unary('-', value=value, loc=self.loc)
         return self.store(expr, res)
 
+    def op_UNARY_POSITIVE(self, inst, value, res):
+        value = self.get(value)
+        expr = ir.Expr.unary('+', value=value, loc=self.loc)
+        return self.store(expr, res)
+
     def op_UNARY_INVERT(self, inst, value, res):
         value = self.get(value)
         expr = ir.Expr.unary('~', value=value, loc=self.loc)
@@ -644,6 +691,10 @@ class Interpreter(object):
     def op_JUMP_IF_TRUE_OR_POP(self, inst, pred):
         self._op_JUMP_IF(inst, pred=pred, iftrue=True)
 
+    def op_RAISE_VARARGS(self, inst, exc):
+        stmt = ir.Raise(exception=self.constants[exc], loc=self.loc)
+        self.current_block.append(stmt)
+
     def _determine_while_condition(self, branches):
         assert branches
         # There is a active syntax block
