@@ -38,8 +38,15 @@ class Loc(object):
             return "%s (%s)" % (self.filename, self.line)
 
     def strformat(self):
-        relpath = os.path.relpath(self.filename)
-        return 'File "%s", line %d' % (relpath, self.line)
+        try:
+            # Try to get a relative path
+            path = os.path.relpath(self.filename)
+        except ValueError:
+            # Fallback to absolute path if error occured in getting the
+            # relative path.
+            # This may happen on windows if the drive is different
+            path = os.path.abspath(self.filename)
+        return 'File "%s", line %d' % (path, self.line)
 
 
 class VarMap(object):
@@ -141,11 +148,6 @@ class Expr(object):
         op = 'getitem'
         return cls(op=op, loc=loc, target=target, index=index)
 
-    @classmethod
-    def getslice(cls, target, start, stop, loc):
-        op = 'getslice'
-        return cls(op=op, loc=loc, target=target, start=start, stop=stop)
-
     def __repr__(self):
         if self.op == 'call':
             args = ', '.join(str(a) for a in self.args)
@@ -174,6 +176,17 @@ class SetItem(Stmt):
         return '%s[%s] = %s' % (self.target, self.index, self.value)
 
 
+class SetAttr(Stmt):
+    def __init__(self, target, attr, value, loc):
+        self.target = target
+        self.attr = attr
+        self.value = value
+        self.loc = loc
+
+    def __repr__(self):
+        return '(%s).%s = %s' % (self.target, self.attr, self.value)
+
+
 class Del(Stmt):
     def __init__(self, value, loc):
         self.value = value
@@ -183,6 +196,17 @@ class Del(Stmt):
         return "del %s" % self.value
 
 
+class Raise(Stmt):
+    is_terminator = True
+
+    def __init__(self, exception, loc):
+        self.exception = exception
+        self.loc = loc
+
+    def __str__(self):
+        return "raise %s" % self.exception
+
+
 class Return(Stmt):
     is_terminator = True
 
@@ -275,6 +299,24 @@ class Var(object):
         return self.name.startswith("$")
 
 
+class Intrinsic(object):
+    """
+    For inserting intrinsic node into the IR
+    """
+
+    def __init__(self, name, type, args):
+        self.name = name
+        self.type = type
+        self.loc = None
+        self.args = args
+
+    def __repr__(self):
+        return 'Intrinsic(%s, %s, %s)' % (self.name, self.type, self.loc)
+
+    def __str__(self):
+        return self.name
+
+
 class Scope(object):
     """
     Attributes
