@@ -5,7 +5,7 @@ import llvm.ee as le
 from llvm import LLVMException
 from numba.config import PYVERSION
 import numba.ctypes_support as ctypes
-from numba import types, utils, cgutils, _numpyadapt, _helperlib
+from numba import types, utils, cgutils, _numpyadapt, _helperlib, assume
 
 _PyNone = ctypes.c_ssize_t(id(None))
 
@@ -24,9 +24,16 @@ def fix_python_api():
     le.dylib_add_symbol("NumbaComplexAdaptor",
                         _helperlib.get_complex_adaptor())
     le.dylib_add_symbol("NumbaNativeError", id(NativeError))
+    le.dylib_add_symbol("NumbaExtractRecordData",
+                        _helperlib.get_extract_record_data())
+    le.dylib_add_symbol("NumbaReleaseRecordBuffer",
+                        _helperlib.get_release_record_buffer())
+    le.dylib_add_symbol("NumbaRecreateRecord",
+                        _helperlib.get_recreate_record())
     le.dylib_add_symbol("PyExc_NameError", id(NameError))
 
 
+
 class PythonAPI(object):
     def __init__(self, context, builder):
         """
@@ -39,6 +46,7 @@ class PythonAPI(object):
         self.module = builder.basic_block.function.module
         # Initialize types
         self.pyobj = self.context.get_argument_type(types.pyobject)
+        self.voidptr = Type.pointer(Type.int(8))
         self.long = Type.int(ctypes.sizeof(ctypes.c_long) * 8)
         self.ulonglong = Type.int(ctypes.sizeof(ctypes.c_ulonglong) * 8)
         self.longlong = self.ulonglong
@@ -96,6 +104,11 @@ class PythonAPI(object):
         fn = self._get_function(fnty, name="PyErr_SetString")
         return self.builder.call(fn, (exctype, msg))
 
+    def err_set_object(self, exctype, excval):
+        fnty = Type.function(Type.void(), [self.pyobj, self.pyobj])
+        fn = self._get_function(fnty, name="PyErr_SetObject")
+        return self.builder.call(fn, (exctype, excval))
+
     def import_module_noblock(self, modname):
         fnty = Type.function(self.pyobj, [self.cstring])
         fn = self._get_function(fnty, name="PyImport_ImportModuleNoBlock")
@@ -222,6 +235,11 @@ class PythonAPI(object):
         fn = self._get_function(fnty, name="PyNumber_Negative")
         return self.builder.call(fn, (obj,))
 
+    def number_positive(self, obj):
+        fnty = Type.function(self.pyobj, [self.pyobj])
+        fn = self._get_function(fnty, name="PyNumber_Positive")
+        return self.builder.call(fn, (obj,))
+
     def number_float(self, val):
         fnty = Type.function(self.pyobj, [self.pyobj])
         fn = self._get_function(fnty, name="PyNumber_Float")
@@ -321,16 +339,23 @@ class PythonAPI(object):
         fn = self._get_function(fnty, name=fname)
         return self.builder.call(fn, [strobj])
 
-    def string_from_string_and_size(self, string):
+    def string_from_string_and_size(self, string, size):
         fnty = Type.function(self.pyobj, [self.cstring, self.py_ssize_t])
         if PYVERSION >= (3, 0):
             fname = "PyUnicode_FromStringAndSize"
         else:
             fname = "PyString_FromStringAndSize"
         fn = self._get_function(fnty, name=fname)
-        cstr = self.context.insert_const_string(self.module, string)
-        sz = self.context.get_constant(types.intp, len(string))
-        return self.builder.call(fn, [cstr, sz])
+        return self.builder.call(fn, [string, size])
+
+    def bytes_from_string_and_size(self, string, size):
+        fnty = Type.function(self.pyobj, [self.cstring, self.py_ssize_t])
+        if PYVERSION >= (3, 0):
+            fname = "PyBytes_FromStringAndSize"
+        else:
+            fname = "PyString_FromStringAndSize"
+        fn = self._get_function(fnty, name=fname)
+        return self.builder.call(fn, [string, size])
 
     def object_str(self, obj):
         fnty = Type.function(self.pyobj, [self.pyobj])
@@ -412,10 +437,14 @@ class PythonAPI(object):
     def print_object(self, obj):
         strobj = self.object_str(obj)
         cstr = self.string_as_string(strobj)
-        fmt = self.context.insert_const_string(self.module, "%s\n")
+        fmt = self.context.insert_const_string(self.module, "%s")
         self.sys_write_stdout(fmt, cstr)
         self.decref(strobj)
 
+    def print_string(self, text):
+        fmt = self.context.insert_const_string(self.module, text)
+        self.sys_write_stdout(fmt)
+
     def get_null_object(self):
         return Constant.null(self.pyobj)
 
@@ -448,7 +477,37 @@ class PythonAPI(object):
         return dictobj
 
     def to_native_arg(self, obj, typ):
-        return self.to_native_value(obj, typ)
+        if isinstance(typ, types.Record):
+            # Generate a dummy integer type that has the size of Py_buffer
+            dummy_py_buffer_type = Type.int(_helperlib.py_buffer_size * 8)
+            # Allocate the Py_buffer
+            py_buffer = cgutils.alloca_once(self.builder, dummy_py_buffer_type)
+
+            # Zero-fill the py_buffer. where the obj field in Py_buffer is NULL
+            # PyBuffer_Release has no effect.
+            zeroed_buffer = lc.Constant.null(dummy_py_buffer_type)
+            self.builder.store(zeroed_buffer, py_buffer)
+
+            buf_as_voidptr = self.builder.bitcast(py_buffer, self.voidptr)
+            ptr = self.extract_record_data(obj, buf_as_voidptr)
+
+            with cgutils.if_unlikely(self.builder,
+                                     cgutils.is_null(self.builder, ptr)):
+                self.builder.ret(ptr)
+
+            ltyp = self.context.get_value_type(typ)
+            val = cgutils.init_record_by_ptr(self.builder, ltyp, ptr)
+
+            def dtor():
+                self.release_record_buffer(buf_as_voidptr)
+
+        else:
+            val = self.to_native_value(obj, typ)
+
+            def dtor():
+                pass
+
+        return val, dtor
 
     def to_native_value(self, obj, typ):
         if isinstance(typ, types.Object) or typ == types.pyobject:
@@ -489,7 +548,7 @@ class PythonAPI(object):
         elif typ in (types.complex128, types.complex64):
             cplxcls = self.context.make_complex(types.complex128)
             cplx = cplxcls(self.context, self.builder)
-            pcplx = cplx._getvalue()
+            pcplx = cplx._getpointer()
             ok = self.complex_adaptor(obj, pcplx)
             failed = cgutils.is_false(self.builder, ok)
 
@@ -561,6 +620,24 @@ class PythonAPI(object):
         elif isinstance(typ, types.Optional):
             return self.from_native_return(val, typ.type)
 
+        elif isinstance(typ, types.Array):
+            return self.from_native_array(typ, val)
+
+        elif isinstance(typ, types.Record):
+            # Note we will create a copy of the record
+            # This is the only safe way.
+            pdata = cgutils.get_record_data(self.builder, val)
+            size = Constant.int(Type.int(), pdata.type.pointee.count)
+            ptr = self.builder.bitcast(pdata, Type.pointer(Type.int(8)))
+            # Note: this will only work for CPU mode
+            #       The following requires access to python object
+            dtype_addr = Constant.int(self.py_ssize_t, id(typ.dtype))
+            dtypeobj = dtype_addr.inttoptr(self.pyobj)
+            return self.recreate_record(ptr, size, dtypeobj)
+
+        elif isinstance(typ, types.UniTuple):
+            return self.from_unituple(typ, val)
+
         raise NotImplementedError(typ)
 
     def to_native_array(self, typ, ary):
@@ -570,14 +647,44 @@ class PythonAPI(object):
         voidptr = Type.pointer(Type.int(8))
         nativearycls = self.context.make_array(typ)
         nativeary = nativearycls(self.context, self.builder)
-        aryptr = nativeary._getvalue()
+        aryptr = nativeary._getpointer()
         ptr = self.builder.bitcast(aryptr, voidptr)
         errcode = self.numba_array_adaptor(ary, ptr)
         failed = cgutils.is_not_null(self.builder, errcode)
         with cgutils.if_unlikely(self.builder, failed):
             # TODO
             self.builder.unreachable()
-        return aryptr
+        return self.builder.load(aryptr)
+
+    def from_native_array(self, typ, ary):
+        assert assume.return_argument_array_only
+        nativearycls = self.context.make_array(typ)
+        nativeary = nativearycls(self.context, self.builder, value=ary)
+        parent = nativeary.parent
+        self.incref(parent)
+        return parent
+
+    def from_unituple(self, typ, val):
+        tuple_val = self.tuple_new(typ.count)
+
+        for i in range(typ.count):
+            item = self.builder.extract_value(val, i)
+            obj = self.from_native_value(item, typ.dtype)
+            self.tuple_setitem(tuple_val, i, obj)
+
+        return tuple_val
+
+    def tuple_new(self, count):
+        fnty = Type.function(self.pyobj, [Type.int()])
+        fn = self._get_function(fnty, name='PyTuple_New')
+        return self.builder.call(fn, [self.context.get_constant(types.int32,
+                                                                count)])
+
+    def tuple_setitem(self, tuple_val, index, item):
+        fnty = Type.function(Type.int(), [self.pyobj, Type.int(), self.pyobj])
+        setitem_fn = self._get_function(fnty, name='PyTuple_SetItem')
+        index = self.context.get_constant(types.int32, index)
+        self.builder.call(setitem_fn, [tuple_val, index, item])
 
     def numba_array_adaptor(self, ary, ptr):
         voidptr = Type.pointer(Type.int(8))
@@ -592,6 +699,23 @@ class PythonAPI(object):
         fn = self._get_function(fnty, name="NumbaComplexAdaptor")
         return self.builder.call(fn, [cobj, cmplx])
 
+    def extract_record_data(self, obj, pbuf):
+        fnty = Type.function(self.voidptr, [self.pyobj,
+                                                         self.voidptr])
+        fn = self._get_function(fnty, name="NumbaExtractRecordData")
+        return self.builder.call(fn, [obj, pbuf])
+
+    def release_record_buffer(self, pbuf):
+        fnty = Type.function(Type.void(), [self.voidptr])
+        fn = self._get_function(fnty, name="NumbaReleaseRecordBuffer")
+        return self.builder.call(fn, [pbuf])
+
+    def recreate_record(self, pdata, size, dtypeaddr):
+        fnty = Type.function(self.pyobj, [Type.pointer(Type.int(8)),
+                                          Type.int(), self.pyobj])
+        fn = self._get_function(fnty, name="NumbaRecreateRecord")
+        return self.builder.call(fn, [pdata, size, dtypeaddr])
+
     def get_module_dict_symbol(self):
         md_pymod = cgutils.MetadataKeyStore(self.module, "python.module")
         pymodname = ".pymodule.dict." + md_pymod.get()
@@ -611,6 +735,12 @@ class PythonAPI(object):
         cstr = self.context.insert_const_string(self.module, msg)
         self.err_set_string(self.native_error_type, cstr)
 
+    def raise_exception(self, exctype, excval):
+        exctypeaddr = self.context.get_constant(types.intp, id(exctype))
+        excvaladdr = self.context.get_constant(types.intp, id(excval))
+        self.err_set_object(exctypeaddr.inttoptr(self.pyobj),
+                            excvaladdr.inttoptr(self.pyobj))
+
     @property
     def native_error_type(self):
         name = "NumbaNativeError"
@@ -618,7 +748,7 @@ class PythonAPI(object):
             return self.module.get_global_variable_named(name)
         except LLVMException:
             return self.module.add_global_variable(self.pyobj.pointee,
-                                                  name=name)
+                                                   name=name)
 
     def raise_missing_global_error(self, name):
         msg = "global name '%s' is not defined" % name
@@ -633,3 +763,8 @@ class PythonAPI(object):
         except LLVMException:
             return self.module.add_global_variable(self.pyobj.pointee,
                                                    name=name)
+
+    def string_from_constant_string(self, string):
+        cstr = self.context.insert_const_string(self.module, string)
+        sz = self.context.get_constant(types.intp, len(string))
+        return self.string_from_string_and_size(cstr, sz)
