@@ -1,10 +1,14 @@
-from llvm.core import Type, Constant
-import llvm.core as lc
+from __future__ import print_function, absolute_import, division
 import math
 from functools import reduce
+
+from llvm.core import Type, Constant
+import llvm.core as lc
+
+from numba import errcode
 from numba import types, typing, cgutils, utils
 from numba.targets.imputils import (builtin, builtin_attr, implement,
-                                    impl_attribute)
+                                    impl_attribute, impl_attribute_generic)
 
 #-------------------------------------------------------------------------------
 
@@ -14,10 +18,10 @@ def make_array(ty):
     nd = ty.ndim
 
     class ArrayTemplate(cgutils.Structure):
-        _fields = [('data',    types.CPointer(dtype)),
-                   ('shape',   types.UniTuple(types.intp, nd)),
+        _fields = [('data', types.CPointer(dtype)),
+                   ('shape', types.UniTuple(types.intp, nd)),
                    ('strides', types.UniTuple(types.intp, nd)),
-                   ('parent',  types.pyobject),]
+                   ('parent', types.pyobject), ]
 
     return ArrayTemplate
 
@@ -134,6 +138,7 @@ def int_sfloordiv_impl(context, builder, sig, args):
     cgutils.guard_zero(context, builder, y)
     return builder.sdiv(x, y)
 
+
 def int_ufloordiv_impl(context, builder, sig, args):
     x, y = args
     cgutils.guard_zero(context, builder, y)
@@ -159,8 +164,14 @@ def int_spower_impl(context, builder, sig, args):
         y = builder.trunc(y, Type.int(32))
     elif y.type.width < 32:
         y = builder.sext(y, Type.int(32))
-    powerfn = lc.Function.intrinsic(module, lc.INTR_POWI, [x.type])
-    return builder.call(powerfn, (x, y))
+
+    if context.implement_powi_as_math_call:
+        undersig = typing.signature(sig.return_type, sig.args[0], types.int32)
+        impl = context.get_function(math.pow, undersig)
+        return impl(builder, (x, y))
+    else:
+        powerfn = lc.Function.intrinsic(module, lc.INTR_POWI, [x.type])
+        return builder.call(powerfn, (x, y))
 
 
 def int_upower_impl(context, builder, sig, args):
@@ -170,8 +181,14 @@ def int_upower_impl(context, builder, sig, args):
         y = builder.trunc(y, Type.int(32))
     elif y.type.width < 32:
         y = builder.zext(y, Type.int(32))
-    powerfn = lc.Function.intrinsic(module, lc.INTR_POWI, [x.type])
-    return builder.call(powerfn, (x, y))
+
+    if context.implement_powi_as_math_call:
+        undersig = typing.signature(sig.return_type, sig.args[0], types.int32)
+        impl = context.get_function(math.pow, undersig)
+        return impl(builder, (x, y))
+    else:
+        powerfn = lc.Function.intrinsic(module, lc.INTR_POWI, [x.type])
+        return builder.call(powerfn, (x, y))
 
 
 def int_power_func_body(context, builder, x, y):
@@ -239,6 +256,7 @@ def int_ugt_impl(context, builder, sig, args):
 def int_uge_impl(context, builder, sig, args):
     return builder.icmp(lc.ICMP_UGE, *args)
 
+
 def int_eq_impl(context, builder, sig, args):
     return builder.icmp(lc.ICMP_EQ, *args)
 
@@ -254,21 +272,12 @@ def int_abs_impl(context, builder, sig, args):
     negated = builder.neg(x)
     return builder.select(ltz, negated, x)
 
+
 def uint_abs_impl(context, builder, sig, args):
     [x] = args
     return x
 
 
-def int_print_impl(context, builder, sig, args):
-    [x] = args
-    py = context.get_python_api(builder)
-    szval = context.cast(builder, x, sig.args[0], types.intp)
-    intobj = py.long_from_ssize_t(szval)
-    py.print_object(intobj)
-    py.decref(intobj)
-    return context.get_dummy_value()
-
-
 def int_shl_impl(context, builder, sig, args):
     [valty, amtty] = sig.args
     [val, amt] = args
@@ -327,6 +336,12 @@ def int_negate_impl(context, builder, sig, args):
         return builder.neg(val)
 
 
+def int_positive_impl(context, builder, sig, args):
+    [typ] = sig.args
+    [val] = args
+    return context.cast(builder, val, typ, sig.return_type)
+
+
 def int_invert_impl(context, builder, sig, args):
     [typ] = sig.args
     [val] = args
@@ -363,7 +378,7 @@ def int_sign_impl(context, builder, sig, args):
     with cgutils.goto_block(builder, bb_pos):
         builder.store(POS, presult)
         builder.branch(bb_exit)
-        
+
     with cgutils.goto_block(builder, bb_neg):
         builder.store(NEG, presult)
         builder.branch(bb_exit)
@@ -387,7 +402,6 @@ for ty in types.integer_domain:
     builtin(implement('==', ty, ty)(int_eq_impl))
     builtin(implement('!=', ty, ty)(int_ne_impl))
 
-    builtin(implement(types.print_type, ty)(int_print_impl))
     builtin(implement('<<', ty, types.uint32)(int_shl_impl))
 
     builtin(implement('&', ty, ty)(int_and_impl))
@@ -395,6 +409,7 @@ for ty in types.integer_domain:
     builtin(implement('^', ty, ty)(int_xor_impl))
 
     builtin(implement('-', ty)(int_negate_impl))
+    builtin(implement('+', ty)(int_positive_impl))
     builtin(implement(types.neg_type, ty)(int_negate_impl))
     builtin(implement('~', ty)(int_invert_impl))
     builtin(implement(types.sign_type, ty)(int_sign_impl))
@@ -586,8 +601,12 @@ def real_mod_impl(context, builder, sig, args):
 def real_power_impl(context, builder, sig, args):
     x, y = args
     module = cgutils.get_module(builder)
-    fn = lc.Function.intrinsic(module, lc.INTR_POW, [y.type])
-    return builder.call(fn, (x, y))
+    if context.implement_powi_as_math_call:
+        imp = context.get_function(math.pow, sig)
+        return imp(builder, args)
+    else:
+        fn = lc.Function.intrinsic(module, lc.INTR_POW, [y.type])
+        return builder.call(fn, (x, y))
 
 
 def real_lt_impl(context, builder, sig, args):
@@ -620,17 +639,6 @@ def real_abs_impl(context, builder, sig, args):
     impl = context.get_function(math.fabs, sig)
     return impl(builder, args)
 
-
-def real_print_impl(context, builder, sig, args):
-    [x] = args
-    py = context.get_python_api(builder)
-    szval = context.cast(builder, x, sig.args[0], types.float64)
-    intobj = py.float_from_double(szval)
-    py.print_object(intobj)
-    py.decref(intobj)
-    return context.get_dummy_value()
-
-
 def real_negate_impl(context, builder, sig, args):
     [typ] = sig.args
     [val] = args
@@ -641,6 +649,12 @@ def real_negate_impl(context, builder, sig, args):
         return builder.neg(val)
 
 
+def real_positive_impl(context, builder, sig, args):
+    [typ] = sig.args
+    [val] = args
+    return context.cast(builder, val, typ, sig.return_type)
+
+
 def real_sign_impl(context, builder, sig, args):
     [x] = args
     POS = Constant.real(x.type, 1)
@@ -670,7 +684,7 @@ def real_sign_impl(context, builder, sig, args):
     with cgutils.goto_block(builder, bb_pos):
         builder.store(POS, presult)
         builder.branch(bb_exit)
-        
+
     with cgutils.goto_block(builder, bb_neg):
         builder.store(NEG, presult)
         builder.branch(bb_exit)
@@ -696,9 +710,9 @@ for ty in types.real_domain:
     builtin(implement('>=', ty, ty)(real_ge_impl))
 
     builtin(implement(types.abs_type, ty)(real_abs_impl))
-    builtin(implement(types.print_type, ty)(real_print_impl))
 
     builtin(implement('-', ty)(real_negate_impl))
+    builtin(implement('+', ty)(real_positive_impl))
     builtin(implement(types.neg_type, ty)(real_negate_impl))
     builtin(implement(types.sign_type, ty)(real_sign_impl))
 
@@ -895,6 +909,11 @@ def complex_negate_impl(context, builder, sig, args):
     return res._getvalue()
 
 
+def complex_positive_impl(context, builder, sig, args):
+    [val] = args
+    return val
+
+
 for ty, cls in zip([types.complex64, types.complex128],
                    [Complex64, Complex128]):
     builtin(implement("+", ty, ty)(complex_add_impl))
@@ -903,6 +922,7 @@ for ty, cls in zip([types.complex64, types.complex128],
     builtin(implement("/?", ty, ty)(complex_div_impl))
     builtin(implement("/", ty, ty)(complex_div_impl))
     builtin(implement("-", ty)(complex_negate_impl))
+    builtin(implement("+", ty)(complex_positive_impl))
     # Complex modulo is deprecated in python3
 
 
@@ -915,6 +935,7 @@ def number_not_impl(context, builder, sig, args):
     istrue = context.cast(builder, val, typ, sig.return_type)
     return builder.not_(istrue)
 
+
 def number_as_bool_impl(context, builder, sig, args):
     [typ] = sig.args
     [val] = args
@@ -933,7 +954,7 @@ builtin(implement('not', types.boolean)(number_not_impl))
 class Slice(cgutils.Structure):
     _fields = [('start', types.intp),
                ('stop', types.intp),
-               ('step', types.intp),]
+               ('step', types.intp), ]
 
 
 @builtin
@@ -1002,6 +1023,7 @@ def slice0_empty_impl(context, builder, sig, args):
 
     return slice3._getvalue()
 
+
 @builtin
 @implement(types.slice_type, types.none, types.none)
 def slice0_none_none_impl(context, builder, sig, args):
@@ -1012,34 +1034,35 @@ def slice0_none_none_impl(context, builder, sig, args):
 
 class RangeState32(cgutils.Structure):
     _fields = [('start', types.int32),
-               ('stop',  types.int32),
-               ('step',  types.int32)]
+               ('stop', types.int32),
+               ('step', types.int32)]
 
 
 class RangeIter32(cgutils.Structure):
-    _fields = [('iter',  types.CPointer(types.int32)),
-               ('stop',  types.int32),
-               ('step',  types.int32),
+    _fields = [('iter', types.CPointer(types.int32)),
+               ('stop', types.int32),
+               ('step', types.int32),
                ('count', types.CPointer(types.int32))]
 
 
 class RangeState64(cgutils.Structure):
     _fields = [('start', types.int64),
-               ('stop',  types.int64),
-               ('step',  types.int64)]
+               ('stop', types.int64),
+               ('step', types.int64)]
 
 
 class RangeIter64(cgutils.Structure):
-    _fields = [('iter',  types.CPointer(types.int64)),
-               ('stop',  types.int64),
-               ('step',  types.int64),
+    _fields = [('iter', types.CPointer(types.int64)),
+               ('stop', types.int64),
+               ('step', types.int64),
                ('count', types.CPointer(types.int64))]
 
 
 def make_unituple_iter(tupiter):
     class UniTupleIter(cgutils.Structure):
-        _fields = [('index',  types.CPointer(types.intp)),
-                   ('tuple',  tupiter.unituple,)]
+        _fields = [('index', types.CPointer(types.intp)),
+                   ('tuple', tupiter.unituple,)]
+
     return UniTupleIter
 
 
@@ -1094,7 +1117,7 @@ def getiter_range_generic(context, builder, iterobj, start, stop, step):
 
     with cgutils.if_unlikely(builder, zero_step):
         # step shouldn't be zero
-        context.return_errcode(builder, 1)
+        context.return_errcode(builder, errcode.ASSERTION_ERROR)
 
     with cgutils.ifelse(builder, sign_differs) as (then, orelse):
         with then:
@@ -1162,7 +1185,6 @@ def itervalid_range32_impl(context, builder, sig, args):
     return gt
 
 
-
 @builtin
 @implement(types.range_type, types.int64)
 def range1_64_impl(context, builder, sig, args):
@@ -1303,8 +1325,7 @@ def getitem_unituple(context, builder, sig, args):
     switch = builder.switch(idx, bbelse, n=tupty.count)
 
     with cgutils.goto_block(builder, bbelse):
-        # TODO: propagate exception to
-        context.return_errcode(builder, 1)
+        context.return_errcode(builder, errcode.OUT_OF_BOUND_ERROR)
 
     lrtty = context.get_value_type(tupty.dtype)
     with cgutils.goto_block(builder, bbend):
@@ -1335,7 +1356,8 @@ def getitem_array1d_intp(context, builder, sig, args):
 
     arystty = make_array(aryty)
     ary = arystty(context, builder, ary)
-    ptr = cgutils.get_item_pointer(builder, aryty, ary, [idx], wraparound=True)
+    ptr = cgutils.get_item_pointer(builder, aryty, ary, [idx],
+                                   wraparound=context.metadata['wraparound'])
     return context.unpack_value(builder, aryty.dtype, ptr)
 
 
@@ -1359,7 +1381,7 @@ def getitem_array1d_slice(context, builder, sig, args):
 
     dataptr = cgutils.get_item_pointer(builder, aryty, ary,
                                        [slicestruct.start],
-                                       wraparound=True)
+                                       wraparound=context.metadata['wraparound'])
 
     retstty = make_array(sig.return_type)
     retary = retstty(context, builder)
@@ -1368,7 +1390,7 @@ def getitem_array1d_slice(context, builder, sig, args):
     retary.shape = cgutils.pack_array(builder, [shape])
 
     stride = cgutils.get_strides_from_slice(builder, aryty.ndim, ary.strides,
-                                             slicestruct, 0)
+                                            slicestruct, 0)
     retary.strides = cgutils.pack_array(builder, [stride])
     retary.data = dataptr
 
@@ -1395,7 +1417,7 @@ def getitem_array_unituple(context, builder, sig, args):
             cgutils.normalize_slice(builder, sl, sh)
         indices = [sl.start for sl in slices]
         dataptr = cgutils.get_item_pointer(builder, aryty, ary, indices,
-                                           wraparound=True)
+                                           wraparound=context.metadata['wraparound'])
         # Build array
         retstty = make_array(sig.return_type)
         retary = retstty(context, builder)
@@ -1416,9 +1438,8 @@ def getitem_array_unituple(context, builder, sig, args):
         indices = cgutils.unpack_tuple(builder, idx, count=len(idxty))
         indices = [context.cast(builder, i, t, types.intp)
                    for t, i in zip(idxty, indices)]
-        # TODO warparound flag
         ptr = cgutils.get_item_pointer(builder, aryty, ary, indices,
-                                       wraparound=True)
+                                       wraparound=context.metadata['wraparound'])
 
         return context.unpack_value(builder, aryty.dtype, ptr)
 
@@ -1456,7 +1477,7 @@ def getitem_array_tuple(context, builder, sig, args):
                 start.append(ind)
 
         dataptr = cgutils.get_item_pointer(builder, aryty, ary, start,
-                                           wraparound=True)
+                                           wraparound=context.metadata['wraparound'])
         # Build array
         retstty = make_array(sig.return_type)
         retary = retstty(context, builder)
@@ -1469,9 +1490,8 @@ def getitem_array_tuple(context, builder, sig, args):
         indices = cgutils.unpack_tuple(builder, idx, count=len(idxty))
         indices = [context.cast(builder, i, t, types.intp)
                    for t, i in zip(idxty, indices)]
-        # TODO warparound flag
         ptr = cgutils.get_item_pointer(builder, aryty, ary, indices,
-                                       wraparound=True)
+                                       wraparound=context.metadata['wraparound'])
 
         return context.unpack_value(builder, aryty.dtype, ptr)
 
@@ -1487,7 +1507,7 @@ def setitem_array1d(context, builder, sig, args):
     ary = arystty(context, builder, ary)
 
     ptr = cgutils.get_item_pointer(builder, aryty, ary, [idx],
-                                   wraparound=True)
+                                   wraparound=context.metadata['wraparound'])
 
     val = context.cast(builder, val, valty, aryty.dtype)
 
@@ -1509,29 +1529,9 @@ def setitem_array_unituple(context, builder, sig, args):
     indices = [context.cast(builder, i, t, types.intp)
                for t, i in zip(idxty, indices)]
     ptr = cgutils.get_item_pointer(builder, aryty, ary, indices,
-                                   wraparound=True)
+                                   wraparound=context.metadata['wraparound'])
     context.pack_value(builder, aryty.dtype, val, ptr)
 
-@builtin
-@implement('setitem', types.Kind(types.Array),
-           types.Kind(types.Tuple), types.Any)
-def setitem_array_tuple(context, builder, sig, args):
-    aryty, idxty, valty = sig.args
-    ary, idx, val = args
-
-    arystty = make_array(aryty)
-    ary = arystty(context, builder, ary)
-
-    # TODO: other than layout
-    indices = cgutils.unpack_tuple(builder, idx, count=len(idxty))
-    indices = [context.cast(builder, i, t, types.intp)
-               for t, i in zip(idxty, indices)]
-    ptr = cgutils.get_item_pointer(builder, aryty, ary, indices,
-                                   wraparound=True)
-    context.pack_value(builder, aryty.dtype, val, ptr)
-
-
-
 
 @builtin
 @implement('setitem', types.Kind(types.Array),
@@ -1548,7 +1548,26 @@ def setitem_array_tuple(context, builder, sig, args):
     indices = [context.cast(builder, i, t, types.intp)
                for t, i in zip(idxty, indices)]
     ptr = cgutils.get_item_pointer(builder, aryty, ary, indices,
-                                   wraparound=True)
+                                   wraparound=context.metadata['wraparound'])
+    context.pack_value(builder, aryty.dtype, val, ptr)
+
+
+@builtin
+@implement('setitem', types.Kind(types.Array),
+           types.Kind(types.Tuple), types.Any)
+def setitem_array_tuple(context, builder, sig, args):
+    aryty, idxty, valty = sig.args
+    ary, idx, val = args
+
+    arystty = make_array(aryty)
+    ary = arystty(context, builder, ary)
+
+    # TODO: other than layout
+    indices = cgutils.unpack_tuple(builder, idx, count=len(idxty))
+    indices = [context.cast(builder, i, t, types.intp)
+               for t, i in zip(idxty, indices)]
+    ptr = cgutils.get_item_pointer(builder, aryty, ary, indices,
+                                   wraparound=context.metadata['wraparound'])
 
     context.pack_value(builder, aryty.dtype, val, ptr)
 
@@ -1598,6 +1617,36 @@ def array_size(context, builder, typ, value):
     return reduce(builder.mul, dims[1:], dims[0])
 
 
+@builtin_attr
+@impl_attribute_generic(types.Array)
+def array_record_getattr(context, builder, typ, value, attr):
+    arrayty = make_array(typ)
+    array = arrayty(context, builder, value)
+
+    rectype = typ.dtype
+    dtype = rectype.typeof(attr)
+    offset = rectype.offset(attr)
+
+    resty = types.Array(dtype, ndim=typ.ndim, layout='A')
+
+    raryty = make_array(resty)
+
+    rary = raryty(context, builder)
+    rary.shape = array.shape
+
+    constoffset = context.get_constant(types.intp, offset)
+    unpackedstrides = cgutils.unpack_tuple(builder, array.strides, typ.ndim)
+    newstrides = [builder.add(s, constoffset) for s in unpackedstrides]
+
+    rary.strides = array.strides
+
+    llintp = context.get_value_type(types.intp)
+    newdata = builder.add(builder.ptrtoint(array.data, llintp), constoffset)
+    newdataptr = builder.inttoptr(newdata, rary.data.type)
+    rary.data = newdataptr
+
+    return rary._getvalue()
+
 #-------------------------------------------------------------------------------
 
 
@@ -1607,6 +1656,7 @@ def caster(restype):
         [val] = args
         [valty] = sig.args
         return context.cast(builder, val, valty, restype)
+
     return _cast
 
 
@@ -1678,6 +1728,7 @@ def min_impl(context, builder, sig, args):
     resty, resval = reduce(domax, typvals)
     return resval
 
+
 @builtin
 @implement(round, types.float32)
 def round_impl_f32(context, builder, sig, args):
@@ -1690,6 +1741,7 @@ def round_impl_f32(context, builder, sig, args):
     assert fn.is_declaration
     return builder.call(fn, args)
 
+
 @builtin
 @implement(round, types.float64)
 def round_impl_f64(context, builder, sig, args):
@@ -1739,3 +1791,17 @@ def complex_impl(context, builder, sig, args):
     cmplx.real = real
     cmplx.imag = imag
     return cmplx._getvalue()
+
+# -----------------------------------------------------------------------------
+
+@builtin_attr
+@impl_attribute(types.Module(math), "pi", types.float64)
+def math_pi_impl(context, builder, typ, value):
+    return context.get_constant(types.float64, math.pi)
+
+
+@builtin_attr
+@impl_attribute(types.Module(math), "e", types.float64)
+def math_e_impl(context, builder, typ, value):
+    return context.get_constant(types.float64, math.e)
+
