@@ -9,7 +9,7 @@ from numba import _dynfunc, _helperlib, config
 from numba.callwrapper import PyCallWrapper
 from .base import BaseContext
 from numba import utils
-from numba.targets import intrinsics, mathimpl, npyimpl
+from numba.targets import intrinsics, mathimpl, npyimpl, operatorimpl, printimpl
 from .options import TargetOptions
 
 
@@ -37,15 +37,19 @@ class CPUContext(BaseContext):
 
         # map math functions
         self.map_math_functions()
+        self.map_numpy_math_functions()
 
         # Add target specific implementations
-        self.insert_func_defn(mathimpl.functions)
-        self.insert_func_defn(npyimpl.functions)
+        self.insert_func_defn(mathimpl.registry.functions)
+        self.insert_func_defn(npyimpl.registry.functions)
+        self.insert_func_defn(operatorimpl.registry.functions)
+        self.insert_func_defn(printimpl.registry.functions)
 
     def build_pass_manager(self):
-        if config.OPT == 3:
+        if 0 < config.OPT <= 3:
             # This uses the same passes for clang -O3
-            pms = lp.build_pass_managers(tm=self.tm, opt=3, loop_vectorize=True,
+            pms = lp.build_pass_managers(tm=self.tm, opt=config.OPT,
+                                         loop_vectorize=True,
                                          fpm=False)
             return pms.pm
         else:
@@ -79,7 +83,6 @@ class CPUContext(BaseContext):
             globalopt
             globaldce
             '''.split()
-
             for p in passes:
                 pm.add(lp.Pass.new(p))
             return pm
@@ -90,6 +93,13 @@ class CPUContext(BaseContext):
         le.dylib_add_symbol("numba.math.srem", _helperlib.get_srem())
         le.dylib_add_symbol("numba.math.udiv", _helperlib.get_udiv())
         le.dylib_add_symbol("numba.math.urem", _helperlib.get_urem())
+        if sys.platform.startswith('win32') and not le.dylib_address_of_symbol('__ftol2'):
+            le.dylib_add_symbol("__ftol2", _helperlib.get_fptoui())
+        elif sys.platform.startswith('linux') and not le.dylib_address_of_symbol('__fixunsdfdi'):
+            le.dylib_add_symbol("__fixunsdfdi", _helperlib.get_fptoui())
+        # Necessary for Python3
+        le.dylib_add_symbol("numba.round", _helperlib.get_round_even())
+        le.dylib_add_symbol("numba.roundf", _helperlib.get_roundf_even())
 
         # windows symbol hacks
         if sys.platform.startswith('win32') and self.is32bit:
@@ -107,6 +117,12 @@ class CPUContext(BaseContext):
                 le.dylib_add_symbol(fname, imp())
                 self.cmath_provider[fname] = 'indirect'
 
+    def map_numpy_math_functions(self):
+        # add the symbols for numpy math to the execution environment.
+        import numba._npymath_exports as npymath
+        for sym in npymath.symbols:
+            le.dylib_add_symbol(*sym)
+
     def dynamic_map_function(self, func):
         name, ptr = self.native_funcs[func]
         le.dylib_add_symbol(name, ptr)
@@ -140,23 +156,31 @@ class CPUContext(BaseContext):
         return cfunc, fnptr
 
     def prepare_for_call(self, func, fndesc):
-        wrapper, api = PyCallWrapper(self, func.module, func, fndesc).build()
+        wrapper, api = PyCallWrapper(self, func.module, func, fndesc,
+                                     exceptions=self.exceptions).build()
         self.optimize(func.module)
 
-        if config.DEBUG:
+        if config.DUMP_OPTIMIZED:
+            print(("OPTIMIZED DUMP %s" % fndesc).center(80,'-'))
             print(func.module)
+            print('=' * 80)
+
+        if config.DUMP_ASSEMBLY:
+            print(("ASSEMBLY %s" % fndesc).center(80, '-'))
             print(self.tm.emit_assembly(func.module))
+            print('=' * 80)
 
         # Map module.__dict__
-        le.dylib_add_symbol(".pymodule.dict." + fndesc.pymod.__name__,
-                            id(fndesc.pymod.__dict__))
+        le.dylib_add_symbol(".pymodule.dict." + fndesc.modname,
+                            id(fndesc.globals))
 
         # Code gen
         self.engine.add_module(func.module)
         baseptr = self.engine.get_pointer_to_function(func)
         fnptr = self.engine.get_pointer_to_function(wrapper)
-        cfunc = _dynfunc.make_function(fndesc.pymod, fndesc.name, fndesc.doc,
-                                       fnptr)
+        cfunc = _dynfunc.make_function(fndesc.lookup_module(),
+                                       fndesc.qualname.split('.')[-1],
+                                       fndesc.doc, fnptr)
 
         if fndesc.native:
             self.native_funcs[cfunc] = fndesc.mangled_name, baseptr
@@ -176,6 +200,9 @@ class CPUContext(BaseContext):
         # remove extra refct api calls
         remove_refct_calls(func)
 
+    def get_abi_sizeof(self, lty):
+        return self.engine.target_data.abi_size(lty)
+
 
 # ----------------------------------------------------------------------------
 # TargetOptions
@@ -184,6 +211,9 @@ class CPUTargetOptions(TargetOptions):
     OPTIONS = {
         "nopython": bool,
         "forceobj": bool,
+        "looplift": bool,
+        "wraparound": bool,
+        "boundcheck": bool,
     }
 
 
