@@ -21,7 +21,7 @@ def implement(func, *argtys):
 def impl_attribute(ty, attr, rtype):
     def wrapper(impl):
         @functools.wraps(impl)
-        def res(context, builder, typ, value):
+        def res(context, builder, typ, value, attr):
             ret = impl(context, builder, typ, value)
             return ret
 
@@ -32,6 +32,19 @@ def impl_attribute(ty, attr, rtype):
     return wrapper
 
 
+def impl_attribute_generic(ty):
+    def wrapper(impl):
+        @functools.wraps(impl)
+        def res(context, builder, typ, value, attr):
+            ret = impl(context, builder, typ, value, attr)
+            return ret
+
+        res.key = (ty, None)
+        return res
+
+    return wrapper
+
+
 def user_function(func, fndesc, libs):
     def imp(context, builder, sig, args):
         func = context.declare_function(cgutils.get_module(builder), fndesc)
