@@ -6,17 +6,14 @@ from __future__ import print_function, absolute_import, division
 import math
 import llvm.core as lc
 from llvm.core import Type
-from numba.targets.imputils import implement, impl_attribute, builtin_attr
+from numba.targets.imputils import implement, Registry
 from numba import types, cgutils, utils
 from numba.typing import signature
+from . import builtins
 
 
-functions = []
-
-
-def register(f):
-    functions.append(f)
-    return f
+registry = Registry()
+register = registry.register
 
 
 def unary_math_int_impl(fn, f64impl):
@@ -82,15 +79,15 @@ def unary_math_extern(fn, f32extern, f64extern):
 
 
 unary_math_intr(math.fabs, lc.INTR_FABS)
-unary_math_intr(math.sqrt, lc.INTR_SQRT)
+#unary_math_intr(math.sqrt, lc.INTR_SQRT)
 unary_math_intr(math.exp, lc.INTR_EXP)
 unary_math_intr(math.log, lc.INTR_LOG)
 unary_math_intr(math.log10, lc.INTR_LOG10)
 unary_math_intr(math.sin, lc.INTR_SIN)
 unary_math_intr(math.cos, lc.INTR_COS)
-unary_math_intr(math.floor, lc.INTR_FLOOR)
-unary_math_intr(math.ceil, lc.INTR_CEIL)
-unary_math_intr(math.trunc, lc.INTR_TRUNC)
+#unary_math_intr(math.floor, lc.INTR_FLOOR)
+#unary_math_intr(math.ceil, lc.INTR_CEIL)
+#unary_math_intr(math.trunc, lc.INTR_TRUNC)
 unary_math_extern(math.log1p, "log1pf", "log1p")
 if utils.PYVERSION > (2, 6):
     unary_math_extern(math.expm1, "expm1f", "expm1")
@@ -104,6 +101,10 @@ unary_math_extern(math.atanh, "atanhf", "atanh")
 unary_math_extern(math.sinh, "sinhf", "sinh")
 unary_math_extern(math.cosh, "coshf", "cosh")
 unary_math_extern(math.tanh, "tanhf", "tanh")
+unary_math_extern(math.ceil, "ceilf", "ceil")
+unary_math_extern(math.floor, "floorf", "floor")
+unary_math_extern(math.sqrt, "sqrtf", "sqrt")
+unary_math_extern(math.trunc, "truncf", "trunc")
 
 
 @register
@@ -306,12 +307,10 @@ unary_math_int_impl(math.degrees, degrees_f64_impl)
 
 # -----------------------------------------------------------------------------
 
-@builtin_attr
-@impl_attribute(types.Module(math), "pi", types.float64)
-def math_pi_impl(context, builder, typ, value):
-    return context.get_constant(types.float64, math.pi)
+for ty in types.unsigned_domain:
+    register(implement(math.pow, types.float64, ty)(builtins.int_upower_impl))
+for ty in types.signed_domain:
+    register(implement(math.pow, types.float64, ty)(builtins.int_spower_impl))
+for ty in types.real_domain:
+    register(implement(math.pow, ty, ty)(builtins.real_power_impl))
 
-@builtin_attr
-@impl_attribute(types.Module(math), "e", types.float64)
-def math_e_impl(context, builder, typ, value):
-    return context.get_constant(types.float64, math.e)
