@@ -30,10 +30,13 @@ if PYVERSION < (3, 0):
 
 
 class TypingError(Exception):
-    def __init__(self, msg, loc):
+    def __init__(self, msg, loc=None):
         self.msg = msg
         self.loc = loc
-        super(TypingError, self).__init__("%s\n%s" % (msg, loc.strformat()))
+        if loc:
+            super(TypingError, self).__init__("%s\n%s" % (msg, loc.strformat()))
+        else:
+            super(TypingError, self).__init__("%s" % (msg,))
 
 
 class TypeVar(object):
@@ -54,8 +57,8 @@ class TypeVar(object):
                 [expect] = list(self.typeset)
                 for ty in types:
                     if self.context.type_compatibility(ty, expect) is None:
-                        raise TypingError("No convertsion from %s to %s" %
-                                          (ty, expect))
+                        raise TypingError("No conversion from %s to %s for "
+                                          "'%s'" % (ty, expect, self.var))
         else:
             self.typeset |= set(types)
 
@@ -63,8 +66,14 @@ class TypeVar(object):
         assert nbefore <= nafter, "Must grow monotonically"
 
     def lock(self, typ):
-        self.typeset = set([typ])
-        self.locked = True
+        if self.locked:
+            [expect] = list(self.typeset)
+            if self.context.type_compatibility(typ, expect) is None:
+                raise TypingError("No conversion from %s to %s for "
+                                  "'%s'" % (typ, expect, self.var))
+        else:
+            self.typeset = set([typ])
+            self.locked = True
 
     def union(self, other):
         self.add_types(*other.typeset)
@@ -212,13 +221,31 @@ class SetItemConstrain(object):
                                   (ty, it, vt), loc=self.loc)
 
 
+class SetAttrConstrain(object):
+    def __init__(self, target, attr, value, loc):
+        self.target = target
+        self.attr = attr
+        self.value = value
+        self.loc = loc
+
+    def __call__(self, context, typevars):
+        targettys = typevars[self.target.name].get()
+        valtys = typevars[self.value.name].get()
+
+        for ty, vt in itertools.product(targettys, valtys):
+            if not context.resolve_setattr(target=ty, attr=self.attr,
+                                           value=vt):
+                raise TypingError("Cannot resolve setattr: (%s).%s = %s" %
+                                  (ty, self.attr, vt), loc=self.loc)
+
+
 class TypeVarMap(dict):
     def set_context(self, context):
         self.context = context
 
     def __getitem__(self, name):
         if name not in self:
-            self[name] = TypeVar(self.context, name)
+            self[name] = TypeVar(self.context, name.split('.', 1)[0])
         return super(TypeVarMap, self).__getitem__(name)
 
     def __setitem__(self, name, value):
@@ -247,6 +274,7 @@ class TypeInferer(object):
         self.usercalls = []
         self.intrcalls = []
         self.setitemcalls = []
+        self.setattrcalls = []
 
     def dump(self):
         print('---- type variables ----')
@@ -260,12 +288,10 @@ class TypeInferer(object):
     def seed_return(self, typ):
         """Seeding of return value is optional.
         """
-        # self.return_type = typ
         for blk in utils.dict_itervalues(self.blocks):
             inst = blk.terminator
             if isinstance(inst, ir.Return):
                 self.typevars[inst.value.name].lock(typ)
-                # self.typevars[inst.value.name].lock()
 
     def build_constrain(self):
         for blk in utils.dict_itervalues(self.blocks):
@@ -292,12 +318,14 @@ class TypeInferer(object):
         typdict = utils.UniqueDict()
         for var, tv in self.typevars.items():
             if len(tv) == 1:
-                typdict[var] = tv.getone()
+                unified = tv.getone()
             elif len(tv) == 0:
                 raise TypeError("Variable %s has no type" % var)
             else:
-                typdict[var] = self.context.unify_types(*tv.get())
-
+                unified = self.context.unify_types(*tv.get())
+            if unified == types.pyobject:
+                raise TypingError("Var '%s' unified to object: %s" % (var, tv))
+            typdict[var] = unified
         retty = self.get_return_type(typdict)
         fntys = self.get_function_types(typdict)
         return typdict, retty, fntys
@@ -316,11 +344,15 @@ class TypeInferer(object):
             calltypes[call] = signature
 
         for call, args, kws in self.usercalls:
-            fnty = typemap[call.func.name]
             args = tuple(typemap[a.name] for a in args)
-            assert not kws
-            signature = self.context.resolve_function_type(fnty, args, ())
-            assert signature is not None, (fnty, args)
+
+            if isinstance(call.func, ir.Intrinsic):
+                signature = call.func.type
+            else:
+                assert not kws
+                fnty = typemap[call.func.name]
+                signature = self.context.resolve_function_type(fnty, args, ())
+                assert signature is not None, (fnty, args)
             calltypes[call] = signature
 
         for inst in self.setitemcalls:
@@ -330,12 +362,14 @@ class TypeInferer(object):
             signature = self.context.resolve_setitem(target, index, value)
             calltypes[inst] = signature
 
-        return calltypes
+        for inst in self.setattrcalls:
+            target = typemap[inst.target.name]
+            attr = inst.attr
+            value = typemap[inst.value.name]
+            signature = self.context.resolve_setattr(target, attr, value)
+            calltypes[inst] = signature
 
-    def guard_return_type(self, ty):
-        if isinstance(ty, types.Array):
-            msg = "Cannot return array in nopython mode"
-            raise TypingError(msg, loc=self.blocks[0].loc)
+        return calltypes
 
     def get_return_type(self, typemap):
         rettypes = set()
@@ -349,14 +383,14 @@ class TypeInferer(object):
             rettypes = rettypes - set([types.none])
             if rettypes:
                 unified = self.context.unify_types(*rettypes)
-                self.guard_return_type(unified)
                 return types.Optional(unified)
             else:
                 return types.none
-        else:
+        elif rettypes:
             unified = self.context.unify_types(*rettypes)
-            self.guard_return_type(unified)
             return unified
+        else:
+            return types.none
 
     def get_state_token(self):
         """The algorithm is monotonic.  It can only grow the typesets.
@@ -370,8 +404,12 @@ class TypeInferer(object):
             self.typeof_assign(inst)
         elif isinstance(inst, ir.SetItem):
             self.typeof_setitem(inst)
+        elif isinstance(inst, ir.SetAttr):
+            self.typeof_setattr(inst)
         elif isinstance(inst, (ir.Jump, ir.Branch, ir.Return, ir.Del)):
             pass
+        elif isinstance(inst, ir.Raise):
+            pass
         else:
             raise NotImplementedError(inst)
 
@@ -381,6 +419,12 @@ class TypeInferer(object):
         self.constrains.append(constrain)
         self.setitemcalls.append(inst)
 
+    def typeof_setattr(self, inst):
+        constrain = SetAttrConstrain(target=inst.target, attr=inst.attr,
+                                     value=inst.value, loc=inst.loc)
+        self.constrains.append(constrain)
+        self.setattrcalls.append(inst)
+
     def typeof_assign(self, inst):
         value = inst.value
         if isinstance(value, ir.Const):
@@ -428,26 +472,43 @@ class TypeInferer(object):
             gvty = self.context.get_global_type(gvar.value)
             self.typevars[target.name].lock(gvty)
             self.assumed_immutables.add(inst)
-        if gvar.name == 'slice' and gvar.value is slice:
+
+        elif gvar.name == 'slice' and gvar.value is slice:
             gvty = self.context.get_global_type(gvar.value)
             self.typevars[target.name].lock(gvty)
             self.assumed_immutables.add(inst)
+
         elif gvar.name == 'len' and gvar.value is len:
             gvty = self.context.get_global_type(gvar.value)
             self.typevars[target.name].lock(gvty)
             self.assumed_immutables.add(inst)
+
         elif gvar.name in ('True', 'False'):
             assert gvar.value in (True, False)
             self.typevars[target.name].lock(types.boolean)
             self.assumed_immutables.add(inst)
+
+        elif gvar.name == 'None':
+            assert gvar.value is None
+            self.typevars[target.name].lock(types.none)
+            self.assumed_immutables.add(inst)
+
+        elif (isinstance(gvar.value, tuple) and
+              all(isinstance(x, int) for x in gvar.value)):
+            gvty = self.context.get_number_type(gvar.value[0])
+            self.typevars[target.name].lock(types.UniTuple(gvty, 2))
+            self.assumed_immutables.add(inst)
+
         elif isinstance(gvar.value, (int, float)):
             gvty = self.context.get_number_type(gvar.value)
             self.typevars[target.name].lock(gvty)
             self.assumed_immutables.add(inst)
+
         elif numpy_support.is_arrayscalar(gvar.value):
             gvty = numpy_support.map_arrayscalar_type(gvar.value)
             self.typevars[target.name].lock(gvty)
             self.assumed_immutables.add(inst)
+
         elif numpy_support.is_array(gvar.value):
             ary = gvar.value
             dtype = numpy_support.from_dtype(ary.dtype)
@@ -455,15 +516,29 @@ class TypeInferer(object):
             gvty = types.Array(dtype, ary.ndim, 'C')
             self.typevars[target.name].lock(gvty)
             self.assumed_immutables.add(inst)
+
         elif ctypes_utils.is_ctypes_funcptr(gvar.value):
             cfnptr = gvar.value
             fnty = ctypes_utils.make_function_type(cfnptr)
             self.typevars[target.name].lock(fnty)
             self.assumed_immutables.add(inst)
+
         elif cffi_support.SUPPORTED and cffi_support.is_cffi_func(gvar.value):
             fnty = cffi_support.make_function_type(gvar.value)
             self.typevars[target.name].lock(fnty)
             self.assumed_immutables.add(inst)
+
+        elif (cffi_support.SUPPORTED and
+                isinstance(gvar.value, cffi_support.ExternCFunction)):
+            fnty = gvar.value
+            self.typevars[target.name].lock(fnty)
+            self.assumed_immutables.add(inst)
+
+        elif type(gvar.value) is type and issubclass(gvar.value,
+                                                     BaseException):
+            self.typevars[target.name].lock(types.exception_type)
+            self.assumed_immutables.add(inst)
+
         else:
             try:
                 gvty = self.context.get_global_type(gvar.value)
@@ -479,7 +554,12 @@ class TypeInferer(object):
 
     def typeof_expr(self, inst, target, expr):
         if expr.op == 'call':
-            self.typeof_call(inst, target, expr)
+            if isinstance(expr.func, ir.Intrinsic):
+                restype = expr.func.type.return_type
+                self.typevars[target.name].add_types(restype)
+                self.usercalls.append((inst.value, expr.args, expr.kws))
+            else:
+                self.typeof_call(inst, target, expr)
         elif expr.op in ('getiter', 'iternext', 'iternextsafe', 'itervalid'):
             self.typeof_intrinsic_call(inst, target, expr.op, expr.value)
         elif expr.op == 'binop':
@@ -501,7 +581,6 @@ class TypeInferer(object):
             constrain = BuildTupleConstrain(target.name, items=expr.items,
                                             loc=inst.loc)
             self.constrains.append(constrain)
-
         else:
             raise NotImplementedError(type(expr), expr)
 
@@ -515,4 +594,4 @@ class TypeInferer(object):
         constrain = IntrinsicCallConstrain(target.name, func, args, (),
                                            loc=inst.loc)
         self.constrains.append(constrain)
-        self.intrcalls.append((inst.value, args, ()))
\ No newline at end of file
+        self.intrcalls.append((inst.value, args, ()))
